#define _TI_ENHANCED_MATH_H 1
#include <math.h>
#include "fft.h"

#define PI 3.141592654


unsigned short log_dual_size( unsigned fft_size)
{
  unsigned short i, nbits;
  
  nbits = 0;
  i = fft_size;
  while( i>1)
  {
    i = i >> 1;
    nbits++;
  }
  return(nbits);
}


void bitrev_index(short ind[], unsigned nbits)
{
  int i,j,k;
  short nbot, ntop, ndiff, n2;
  
  nbot = nbits >> 1;
  ndiff = nbits & 1;
  ntop = nbot + ndiff;
  n2 = 1 << ntop;
  
    ind[0] = 0;
    for( i=1, j=n2/2+1; i<n2-1; i++)
    {
      ind[i] = j - 1;
      for( k = n2/2; k<j; k /= 2)
        j -= k;
      j += k;
    }
    ind[n2-1] = n2 - 1;
}


void calc_sine_coeffs(short ptr1[], unsigned fft_size)
{
  unsigned i1,i2;
  float delta; 
  
  delta = 2*PI/(float)fft_size;
  if(ptr1)
  {
    for( i1=0, i2=0;  i2<fft_size;  i1++, i2+=2)
    {
      // see DSPLIB
      ptr1[i2]   = (short)roundf( -32767.0 * cosf( (float)i1*delta) );
      ptr1[i2+1] = (short)roundf( -32767.0 * sinf( (float)i1*delta) );
      
      // conjugated version for ifft
      ptr1[fft_size+i2]   =  ptr1[i2];
      ptr1[fft_size+i2+1] = -ptr1[i2+1];
    }
  }
}


void fft_init( fft16Params *fft16, unsigned fft_size)
{
  fft16->fft_size=fft_size;
  fft16->log_dual_sz = log_dual_size( fft16->fft_size);
  bitrev_index(fft16->br_ind ,fft16->log_dual_sz);
  calc_sine_coeffs(fft16->sine_coeffs, fft16->fft_size);
}


void fft( fft16Params *fft16,short sigbuf[])
{
  unsigned i;
 for(i=0; i<2*fft16->fft_size; i++)
       sigbuf[i] = sigbuf[i] >> (fft16->log_dual_sz>1 ); /* normalize by fft_size*/

  DSP_radix2( fft16->fft_size, sigbuf, fft16->sine_coeffs);  
  DSP_bitrev_cplx( sigbuf, fft16->br_ind, fft16->fft_size); 
}


void ifft( fft16Params *fft16,short sigbuf[])
{
  unsigned i;
  
  DSP_radix2( fft16->fft_size, sigbuf, fft16->sine_coeffs+fft16->fft_size);
  DSP_bitrev_cplx(sigbuf, fft16->br_ind,fft16->fft_size);  
  for(i=0; i<2*fft16->fft_size; i++)
       sigbuf[i] = sigbuf[i] >> (fft16->log_dual_sz>1 ); /* normalize by fft_size*/
}

void hamming_window( short buf[], unsigned short size)
{
	unsigned short i=0;
	for(i=0;i<size;i++)
		buf[i]=17694-(short)(32767.0*0.46*cosf(2*PI*i/size));
	
}

 
